@@ -14,7 +14,4 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-"""
-The Aerostructure MDO test case
-
-"""
+"""The Aerostructure MDO test case."""
